@@ -50,6 +50,7 @@
 #define DISABLECONTEXTMENU	"disablecontextmenu"
 #define DISABLEPAUSEONCLICK	"disable_pause_on_click"
 #define DISABLEANIMATION	"disable_animation"
+#define AUTOHIDETIMEOUT	"auto_hide_timeout"
 #define METADATACODEPAGE	"metadatacodepage"
 #define SUBTITLEFONT	"subtitlefont"
 #define SUBTITLESCALE	"subtitlescale"
@@ -105,7 +106,9 @@ typedef enum {
 
 typedef enum {
     NO_ERROR,
-    ERROR_RETRY_WITH_PLAYLIST
+    ERROR_RETRY_WITH_PLAYLIST,
+    ERROR_RETRY_WITH_HTTP,
+	ERROR_RETRY_WITH_MMSHTTP
 } PLAYBACK_ERROR;
 
 PLAYSTATE state;
@@ -126,7 +129,7 @@ typedef struct _IdleData {
     gint streaming;
     gchar progress_text[1024];
     gdouble volume;
-    gint mute;
+    gboolean mute;
     gchar vol_tooltip[128];
     gint x;
     gint y;
@@ -170,7 +173,7 @@ typedef struct _IdleData {
     GFile *tmp;
     GCancellable *cancel;
     GMutex *caching;
-	GCond *caching_complete;
+    GCond *caching_complete;
 #endif
 } IdleData;
 
@@ -283,6 +286,7 @@ gchar *alang;
 gchar *slang;
 gchar *metadata_codepage;
 gint volume;
+gboolean use_volume_option;
 gboolean vertical_layout;
 gboolean playlist_visible;
 gboolean details_visible;
@@ -293,6 +297,7 @@ gint restore_pane;
 gboolean disable_ass;
 gboolean disable_embeddedfonts;
 gboolean disable_animation;
+gint auto_hide_timeout;
 gchar *subtitlefont;
 gdouble subtitle_scale;
 gchar *subtitle_codepage;
