@@ -71,12 +71,14 @@ gint detect_playlist(gchar * uri)
         playlist = 1;
 //      } else if (g_strncasecmp(filename,"dvdnav://",9) == 0) {
 //              playlist = 1;
+    } else if (device_name(uri)) {
+        playlist = 0;
     } else {
 
 #ifdef GIO_ENABLED
         if (!streaming_media(uri)) {
-			if (verbose)
-				printf("opening playlist\n");
+            if (verbose)
+                printf("opening playlist\n");
             file = g_file_new_for_uri(uri);
             path = get_path(uri);
             input = g_file_read(file, NULL, NULL);
@@ -289,8 +291,8 @@ gint parse_playlist(gchar * uri)
         gtk_recent_manager_add_item(recent_manager, uri);
     }
 #endif
-	if (verbose)
-		printf("parse playlist = %i\n", ret);
+    if (verbose)
+        printf("parse playlist = %i\n", ret);
     return ret;
 }
 
@@ -525,7 +527,11 @@ gint parse_cdda(gchar * filename)
         return 0;
     } else {
         playlist = 0;
-        av[ac++] = g_strdup_printf("mplayer");
+        if (mplayer_bin == NULL || !g_file_test(mplayer_bin, G_FILE_TEST_EXISTS)) {
+            av[ac++] = g_strdup_printf("mplayer");
+        } else {
+            av[ac++] = g_strdup_printf("%s", mplayer_bin);
+        }
         av[ac++] = g_strdup_printf("-vo");
         av[ac++] = g_strdup_printf("null");
         av[ac++] = g_strdup_printf("-ao");
@@ -682,7 +688,11 @@ gint parse_dvd(gchar * filename)
     if (g_strncasecmp(filename, "dvd://", strlen("dvd://")) == 0) {     // || g_strncasecmp(filename,"dvdnav://",strlen("dvdnav://")) == 0) {
         playlist = 0;
         // run mplayer and try to get the first frame and convert it to a jpeg
-        av[ac++] = g_strdup_printf("mplayer");
+        if (mplayer_bin == NULL || !g_file_test(mplayer_bin, G_FILE_TEST_EXISTS)) {
+            av[ac++] = g_strdup_printf("mplayer");
+        } else {
+            av[ac++] = g_strdup_printf("%s", mplayer_bin);
+        }
         av[ac++] = g_strdup_printf("-vo");
         av[ac++] = g_strdup_printf("null");
         av[ac++] = g_strdup_printf("-ao");
@@ -936,6 +946,12 @@ gboolean streaming_media(gchar * uri)
         ret = FALSE;
     } else if (g_ascii_strncasecmp(uri, "http://", strlen("http://")) == 0) {
         ret = TRUE;
+    } else if (g_ascii_strncasecmp(uri, "mmst://", strlen("mmst://")) == 0) {
+        ret = TRUE;
+    } else if (g_ascii_strncasecmp(uri, "mms://", strlen("mms://")) == 0) {
+        ret = TRUE;
+    } else if (g_ascii_strncasecmp(uri, "mmshttp://", strlen("mmshttp://")) == 0) {
+        ret = TRUE;
     } else {
 #ifdef GIO_ENABLED
         file = g_file_new_for_uri(uri);
@@ -980,6 +996,8 @@ gboolean device_name(gchar * filename)
         ret = TRUE;
     } else if (g_ascii_strncasecmp(filename, "tv://", strlen("tv://")) == 0) {
         ret = TRUE;
+    } else if (g_ascii_strncasecmp(filename, "dvb://", strlen("dvb://")) == 0) {
+        ret = TRUE;
     } else if (g_ascii_strncasecmp(filename, "vcd://", strlen("vcd://")) == 0) {
         ret = TRUE;
     } else {
@@ -1038,6 +1056,9 @@ MetaData *get_metadata(gchar * uri)
 #endif
     if (device_name(uri)) {
         name = g_strdup(uri);
+        if (ret == NULL)
+            ret = (MetaData *) g_new0(MetaData, 1);
+
     } else {
 #ifdef GIO_ENABLED
         file = g_file_new_for_uri(uri);
@@ -1053,11 +1074,15 @@ MetaData *get_metadata(gchar * uri)
 
     if (name == NULL)
         return NULL;
-	
-	if (verbose)
-		printf("getting file metadata for %s\n", name);
 
-    av[ac++] = g_strdup_printf("mplayer");
+    if (verbose)
+        printf("getting file metadata for %s\n", name);
+
+    if (mplayer_bin == NULL || !g_file_test(mplayer_bin, G_FILE_TEST_EXISTS)) {
+        av[ac++] = g_strdup_printf("mplayer");
+    } else {
+        av[ac++] = g_strdup_printf("%s", mplayer_bin);
+    }
     av[ac++] = g_strdup_printf("-vo");
     av[ac++] = g_strdup_printf("null");
     av[ac++] = g_strdup_printf("-ao");
@@ -1077,7 +1102,14 @@ MetaData *get_metadata(gchar * uri)
 
     error = NULL;
 
-    g_spawn_sync(NULL, av, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, &out, &err, &exit_status, &error);
+    if (g_strncasecmp(name, "dvb://", strlen("dvb://")) == 0
+        || g_strncasecmp(name, "tv://", strlen("tv://")) == 0) {
+        if (verbose)
+            printf("Skipping gathering metadata for TV channels\n");
+    } else {
+        g_spawn_sync(NULL, av, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, &out, &err, &exit_status,
+                     &error);
+    }
 
     for (i = 0; i < ac; i++) {
         g_free(av[i]);
@@ -1093,9 +1125,13 @@ MetaData *get_metadata(gchar * uri)
             g_free(err);
         return NULL;
     }
-    output = g_strsplit(out, "\n", 0);
+    if (out != NULL) {
+        output = g_strsplit(out, "\n", 0);
+    } else {
+        output = NULL;
+    }
     ac = 0;
-    while (output[ac] != NULL) {
+    while (output != NULL && output[ac] != NULL) {
         lower = g_ascii_strdown(output[ac], -1);
         if (strstr(output[ac], "_LENGTH") != NULL
             && (g_strncasecmp(name, "dvdnav://", strlen("dvdnav://")) != 0
@@ -1191,6 +1227,11 @@ MetaData *get_metadata(gchar * uri)
         title = g_strdup_printf("%s", localtitle);
     }
 
+    if (title == NULL && g_strncasecmp(name, "dvb://", strlen("dvb://")) == 0) {
+        localtitle = g_strrstr(name, "/") + 1;
+        title = g_strdup_printf("%s", localtitle);
+    }
+
     if (title == NULL && g_strncasecmp(name, "dvdnav://", strlen("dvdnav://")) == 0) {
         title = g_strdup_printf("DVD");
     }
@@ -1246,7 +1287,11 @@ gint get_bitrate(gchar * name)
     if (!g_file_test(name, G_FILE_TEST_EXISTS))
         return 0;
 
-    av[ac++] = g_strdup_printf("mplayer");
+    if (mplayer_bin == NULL || !g_file_test(mplayer_bin, G_FILE_TEST_EXISTS)) {
+        av[ac++] = g_strdup_printf("mplayer");
+    } else {
+        av[ac++] = g_strdup_printf("%s", mplayer_bin);
+    }
     av[ac++] = g_strdup_printf("-vo");
     av[ac++] = g_strdup_printf("null");
     av[ac++] = g_strdup_printf("-ao");
@@ -1377,31 +1422,40 @@ gint get_bitrate(gchar * name)
 }
 
 
-gboolean add_item_to_playlist(gchar * uri, gint playlist)
+gboolean add_item_to_playlist(const gchar * uri, gint playlist)
 {
     GtkTreeIter localiter;
+	gchar *local_uri;
     gchar *unescaped = NULL;
     MetaData *data = NULL;
 
-	if (verbose)
-		printf("adding %s to playlist\n", uri);
-    if (!device_name(uri) && !streaming_media(uri)) {
+    if (verbose)
+        printf("adding %s to playlist\n", uri);
+	local_uri = strdup(uri);
+    if (!device_name(local_uri) && !streaming_media(local_uri)) {
         if (playlist) {
             data = (MetaData *) g_new0(MetaData, 1);
             data->title = g_strdup_printf("%s", uri);
         } else {
-            data = get_metadata(uri);
+            data = get_metadata(local_uri);
         }
     } else if (g_ascii_strncasecmp(uri, "cdda://", strlen("cdda://")) == 0) {
         data = (MetaData *) g_new0(MetaData, 1);
         data->title = g_strdup_printf("CD Track %s", uri + strlen("cdda://"));
-    } else if (device_name(uri)) {
+    } else if (device_name(local_uri)) {
         if (g_strncasecmp(uri, "dvdnav://", strlen("dvdnav://") == 0)) {
             loop = 1;
         }
-        data = get_metadata(uri);
+        data = get_metadata(local_uri);
 
     } else {
+
+        if (g_str_has_prefix(uri, "http://")) {
+            unescaped = switch_protocol(uri,"mmshttp");
+            g_free(local_uri);
+            local_uri = g_strdup(unescaped);
+            g_free(unescaped);
+        }
 #ifdef GIO_ENABLED
         unescaped = g_uri_unescape_string(uri, NULL);
 #else
@@ -1414,7 +1468,7 @@ gboolean add_item_to_playlist(gchar * uri, gint playlist)
 
     if (data) {
         gtk_list_store_append(playliststore, &localiter);
-        gtk_list_store_set(playliststore, &localiter, ITEM_COLUMN, uri,
+        gtk_list_store_set(playliststore, &localiter, ITEM_COLUMN, local_uri,
                            DESCRIPTION_COLUMN, data->title,
                            COUNT_COLUMN, 0,
                            PLAYLIST_COLUMN, playlist,
@@ -1430,7 +1484,7 @@ gboolean add_item_to_playlist(gchar * uri, gint playlist)
 
 
         gtk_list_store_append(nonrandomplayliststore, &localiter);
-        gtk_list_store_set(nonrandomplayliststore, &localiter, ITEM_COLUMN, uri,
+        gtk_list_store_set(nonrandomplayliststore, &localiter, ITEM_COLUMN, local_uri,
                            DESCRIPTION_COLUMN, data->title,
                            COUNT_COLUMN, 0,
                            PLAYLIST_COLUMN, playlist,
@@ -1444,7 +1498,7 @@ gboolean add_item_to_playlist(gchar * uri, gint playlist)
                            LENGTH_VALUE_COLUMN, data->length_value,
                            VIDEO_WIDTH_COLUMN, data->width, VIDEO_HEIGHT_COLUMN, data->height, -1);
 
-        set_item_add_info(uri);
+        set_item_add_info(local_uri);
         g_free(data->demuxer);
         g_free(data->title);
         g_free(data->artist);
@@ -1454,8 +1508,10 @@ gboolean add_item_to_playlist(gchar * uri, gint playlist)
         g_free(data->audio_codec);
         g_free(data->video_codec);
         g_free(data);
+		g_free(local_uri);
         return TRUE;
     } else {
+		g_free(local_uri);
         return FALSE;
     }
 
@@ -2140,7 +2196,7 @@ void cache_callback(goffset current_num_bytes, goffset total_num_bytes, gpointer
     idledata->cachepercent = (gfloat) current_num_bytes / (gfloat) total_num_bytes;
     g_idle_add(set_progress_value, idledata);
     if (current_num_bytes > (cache_size * 1024))
-		g_cond_signal(idledata->caching_complete);
+        g_cond_signal(idledata->caching_complete);
 
 }
 
@@ -2148,7 +2204,7 @@ void ready_callback(GObject * source_object, GAsyncResult * res, gpointer data)
 {
     g_object_unref(idledata->tmp);
     g_object_unref(idledata->src);
-	g_cond_signal(idledata->caching_complete);
+    g_cond_signal(idledata->caching_complete);
 }
 #endif
 
@@ -2190,8 +2246,8 @@ gchar *get_localfile_from_uri(gchar * uri)
                 g_file_copy_async(idledata->src, idledata->tmp, G_FILE_COPY_NONE,
                                   G_PRIORITY_DEFAULT, idledata->cancel, cache_callback, NULL,
                                   ready_callback, NULL);
-				g_cond_wait(idledata->caching_complete,idledata->caching);
-				g_mutex_unlock(idledata->caching);
+                g_cond_wait(idledata->caching_complete, idledata->caching);
+                g_mutex_unlock(idledata->caching);
                 idledata->tmpfile = TRUE;
             }
         } else {
@@ -2209,6 +2265,11 @@ gboolean is_uri_dir(gchar * uri)
 {
     gboolean result = FALSE;
 
+    if (uri == NULL)
+        return FALSE;
+    if (device_name(uri))
+        return result;
+
 #ifdef GIO_ENABLED
     GFile *file;
     GFileInfo *info;
@@ -2240,6 +2301,8 @@ gboolean uri_exists(gchar * uri)
 
     if (uri == NULL)
         return FALSE;
+    if (device_name(uri))
+        return result;
 
 #ifdef GIO_ENABLED
     GFile *file;
@@ -2463,7 +2526,7 @@ gpointer get_cover_art(gpointer data)
 {
     gchar *url;
     gchar *path;
-	gchar *p;
+    gchar *p;
     gchar *cache_file;
     gboolean local_artist = FALSE;
     gboolean local_album = FALSE;
@@ -2518,7 +2581,7 @@ gpointer get_cover_art(gpointer data)
             }
             if (url == NULL) {
                 if ((p = strstr(metadata->title, " - ")) != NULL) {
-					p[0] = '\0';
+                    p[0] = '\0';
                     g_free(path);
                     path =
                         g_strdup_printf("%s/gnome-mplayer/cover_art/Unknown",
@@ -2558,14 +2621,16 @@ gpointer get_cover_art(gpointer data)
                     }
 
                     art = fopen(cache_file, "wb");
-                    curl = curl_easy_init();
-                    if (curl) {
-                        curl_easy_setopt(curl, CURLOPT_URL, url);
-                        curl_easy_setopt(curl, CURLOPT_WRITEDATA, art);
-                        curl_easy_perform(curl);
-                        curl_easy_cleanup(curl);
+                    if (art) {
+                        curl = curl_easy_init();
+                        if (curl) {
+                            curl_easy_setopt(curl, CURLOPT_URL, url);
+                            curl_easy_setopt(curl, CURLOPT_WRITEDATA, art);
+                            curl_easy_perform(curl);
+                            curl_easy_cleanup(curl);
+                        }
+                        fclose(art);
                     }
-                    fclose(art);
                     // printf("cover art url is %s\n",url);
                     g_free(url);
                     art_found = TRUE;
@@ -2641,3 +2706,77 @@ gchar *gmp_tempname(gchar * path, const gchar * name_template)
 
     return result;
 }
+
+
+/* 
+	older mplayer binaries do not have the -volume option
+	So rather than handleing several hundred bug reports since
+	Ubuntu has some older mplayer binaries.
+	we'll try and detect if we can use it or not.
+*/
+gboolean detect_volume_option()
+{
+    gchar *av[255];
+    gint ac = 0, i;
+    gchar **output;
+    GError *error;
+    gint exit_status;
+    gchar *out = NULL;
+    gchar *err = NULL;
+    gboolean ret = TRUE;
+
+    if (mplayer_bin == NULL || !g_file_test(mplayer_bin, G_FILE_TEST_EXISTS)) {
+        av[ac++] = g_strdup_printf("mplayer");
+    } else {
+        av[ac++] = g_strdup_printf("%s", mplayer_bin);
+    }
+    av[ac++] = g_strdup_printf("-volume");
+    av[ac++] = g_strdup_printf("100");
+    av[ac] = NULL;
+
+    error = NULL;
+
+    g_spawn_sync(NULL, av, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, &out, &err, &exit_status, &error);
+
+    for (i = 0; i < ac; i++) {
+        g_free(av[i]);
+    }
+
+    if (error != NULL) {
+        printf("Error when running: %s\n", error->message);
+        g_error_free(error);
+        error = NULL;
+        if (out != NULL)
+            g_free(out);
+        if (err != NULL)
+            g_free(err);
+        return FALSE;
+    }
+    output = g_strsplit(err, "\n", 0);
+    ac = 0;
+    while (output[ac] != NULL) {
+        if (g_strncasecmp(output[ac], "Unknown option", strlen("Unknown option")) == 0) {
+            ret = FALSE;
+        }
+        ac++;
+    }
+    g_strfreev(output);
+    g_free(out);
+    g_free(err);
+
+    if (!ret)
+        printf(_
+               ("You might want to consider upgrading mplayer to a newer version, -volume option not supported\n"));
+    return ret;
+}
+
+gchar *switch_protocol(const gchar *uri, gchar *new_protocol) {
+	gchar *p;
+	
+	p = g_strrstr(uri,"://");
+	
+	if (p != NULL)
+		return g_strdup_printf("%s%s",new_protocol,p);
+	else
+		return NULL;
+}
