@@ -77,7 +77,7 @@ gboolean update_mplayer_config();
 gboolean send_command(gchar * command, gboolean retain_pause);
 gboolean streaming_media(gchar * filename);
 gboolean device_name(gchar * filename);
-gboolean add_item_to_playlist(gchar * itemname, gint playlist);
+gboolean add_item_to_playlist(const gchar * uri, gint playlist);
 gboolean next_item_in_playlist(GtkTreeIter * iter);
 void copy_playlist(GtkListStore * source, GtkListStore * dest);
 void randomize_playlist(GtkListStore * store);
@@ -98,6 +98,7 @@ void release_preference_store();
 gchar *get_localfile_from_uri(gchar * uri);
 gboolean is_uri_dir(gchar * uri);
 gboolean uri_exists(gchar * uri);
+gchar *switch_protocol(const gchar *uri, gchar *new_protocol);
 
 #ifdef HAVE_GPOD
 gchar *find_gpod_mount_point();
@@ -108,5 +109,6 @@ gchar *get_cover_art_url(gchar * artist, gchar * title, gchar * album);
 gpointer get_cover_art(gpointer data);
 
 gchar *gmp_tempname(gchar * path, const gchar * name_template);
+gboolean detect_volume_option();
 
 #endif                          // _SUPPORT_H
