@@ -70,6 +70,8 @@ gboolean play(void *data)
     PlayData *p = (PlayData *) data;
 
     if (ok_to_play && p != NULL) {
+        gtk_list_store_set(playliststore, &iter, PLAYLIST_COLUMN, p->playlist, ITEM_COLUMN,
+                           p->uri);
         play_iter(&iter);
     }
     g_free(p);
@@ -85,10 +87,7 @@ gboolean thread_complete(GIOChannel * source, GIOCondition condition, gpointer d
     state = QUIT;
     g_source_remove(watch_in_id);
     g_source_remove(watch_err_id);
-    if (verbose > 1)
-        printf("thread_complete, unlocking thread running\n");
-    // g_mutex_unlock(thread_running);
-	g_cond_signal(mplayer_complete_cond);
+    g_cond_signal(mplayer_complete_cond);
     return FALSE;
 }
 
@@ -119,7 +118,7 @@ gboolean thread_reader_error(GIOChannel * source, GIOCondition condition, gpoint
         g_source_remove(watch_in_id);
         g_source_remove(watch_in_hup_id);
         //g_mutex_unlock(thread_running);
-		g_cond_signal(mplayer_complete_cond);
+        g_cond_signal(mplayer_complete_cond);
         return FALSE;
     }
 
@@ -144,10 +143,16 @@ gboolean thread_reader_error(GIOChannel * source, GIOCondition condition, gpoint
             strstr(mplayer_output->str, "registry file") == NULL) {
             // error_msg = g_strdup(mplayer_output->str);
             if (strstr(mplayer_output->str, "<") == NULL && strstr(mplayer_output->str, ">") == NULL
-                && idledata->streaming == FALSE)
+                && idledata->streaming == FALSE) {
                 error_msg =
                     g_strdup_printf(_("Failed to open %s"),
                                     mplayer_output->str + strlen("Failed to open "));
+			}
+		
+			if (strstr(mplayer_output->str, "mms://") != NULL && idledata->streaming) {
+				dontplaynext = TRUE;
+				playback_error = ERROR_RETRY_WITH_MMSHTTP;
+			}
         }
     }
 
@@ -227,7 +232,7 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
         g_source_remove(watch_err_id);
         g_source_remove(watch_in_hup_id);
         //g_mutex_unlock(thread_running);
-		g_cond_signal(mplayer_complete_cond);
+        g_cond_signal(mplayer_complete_cond);
         return FALSE;
     }
     mplayer_output = g_string_new("");
@@ -245,7 +250,7 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
                 g_idle_add(set_stop, idledata);
                 state = QUIT;
                 //g_mutex_unlock(thread_running);
-				g_cond_signal(mplayer_complete_cond);
+                g_cond_signal(mplayer_complete_cond);
                 g_error_free(error);
                 error = NULL;
                 return FALSE;
@@ -255,7 +260,7 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
             g_idle_add(set_stop, idledata);
             state = QUIT;
             //g_mutex_unlock(thread_running);
-			g_cond_signal(mplayer_complete_cond);
+            g_cond_signal(mplayer_complete_cond);
             error = NULL;
             return FALSE;
         }
@@ -263,7 +268,7 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
 
     if (strstr(mplayer_output->str, "ID_SIGNAL") != NULL) {
         //g_mutex_unlock(thread_running);
-		g_cond_signal(mplayer_complete_cond);
+        g_cond_signal(mplayer_complete_cond);
         g_string_free(mplayer_output, TRUE);
         state = QUIT;
         return FALSE;
@@ -287,7 +292,7 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
         g_idle_add(set_stop, idledata);
         g_string_free(mplayer_output, TRUE);
         //g_mutex_unlock(thread_running);
-		g_cond_signal(mplayer_complete_cond);	
+        g_cond_signal(mplayer_complete_cond);
         return FALSE;
     }
 
@@ -382,12 +387,14 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
     if (strstr(mplayer_output->str, "ANS_volume") != 0) {
         buf = strstr(mplayer_output->str, "ANS_volume");
         sscanf(buf, "ANS_volume=%i", &volume);
-        idledata->volume = volume;
-        idledata->mute = 0;
-        buf = g_strdup_printf(_("Volume %i%%"), volume);
-        g_strlcpy(idledata->vol_tooltip, buf, 128);
-        g_idle_add(set_volume_tip, idledata);
-        g_free(buf);
+        if (!idledata->mute) {
+            idledata->volume = volume;
+            idledata->mute = 0;
+            buf = g_strdup_printf(_("Volume %i%%"), volume);
+            g_strlcpy(idledata->vol_tooltip, buf, 128);
+            g_idle_add(set_volume_tip, idledata);
+            g_free(buf);
+        }
     }
 
     if (strstr(mplayer_output->str, "ANS_chapters") != 0) {
@@ -756,7 +763,7 @@ gpointer launch_player(gpointer data)
             argv[arg++] = g_strdup_printf("ffmpeg12mc");
         } else {
             argv[arg++] = g_strdup_printf("-vo");
-            argv[arg++] = g_strdup_printf("xv");
+            argv[arg++] = g_strdup_printf("xv,x11");
             if (!disable_deinterlace) {
                 argv[arg++] = g_strdup_printf("-vf-pre");
                 argv[arg++] = g_strdup_printf("yadif,softskip,scale");
@@ -782,7 +789,7 @@ gpointer launch_player(gpointer data)
             argv[arg++] = g_strdup_printf("ffvc1vdpau");
         } else {
             argv[arg++] = g_strdup_printf("-vo");
-            argv[arg++] = g_strdup_printf("xv");
+            argv[arg++] = g_strdup_printf("vdpau,xv,x11");
             if (!disable_deinterlace) {
                 argv[arg++] = g_strdup_printf("-vf-pre");
                 argv[arg++] = g_strdup_printf("yadif,softskip,scale");
@@ -799,6 +806,11 @@ gpointer launch_player(gpointer data)
     if (softvol)
         argv[arg++] = g_strdup_printf("-softvol");
 
+    if (use_volume_option) {
+        argv[arg++] = g_strdup_printf("-volume");
+        argv[arg++] = g_strdup_printf("%i", (gint) idledata->volume);
+    }
+
     if (mixer != NULL && strlen(mixer) > 0) {
         if (ao == NULL || (ao != NULL && g_ascii_strncasecmp(ao, "alsa", 4) == 0)) {
             argv[arg++] = g_strdup_printf("-mixer-channel");
@@ -948,6 +960,19 @@ gpointer launch_player(gpointer data)
         argv[arg++] = g_strdup_printf("%s", idledata->device);
     }
 
+    /* 
+       This is here in order to be able to switch the
+       audio track in OGM files while playing, see
+       http://lists.mplayerhq.hu/pipermail/mplayer-users/2007-February/065316.html 
+     */
+    filename = g_utf8_strdown(threaddata->filename, -1);
+    if (strstr(filename, ".ogm")) {
+        argv[arg++] = g_strdup_printf("-demuxer");
+        argv[arg++] = g_strdup_printf("lavf");
+    }
+    g_free(filename);
+
+
 /*	This is needed when lavc is the default decoder, but mkv is the default as of 9/9/08
 	filename = g_utf8_strdown(threaddata->filename,-1);
 	if (strstr(filename,".mkv")) {
@@ -1013,14 +1038,7 @@ gpointer launch_player(gpointer data)
         channel_in = g_io_channel_unix_new(std_in);
         channel_out = g_io_channel_unix_new(std_out);
         channel_err = g_io_channel_unix_new(std_err);
-/*        
-		flags = g_io_channel_get_flags(channel_in);
-        flags |= G_IO_FLAG_NONBLOCK;
-        g_io_channel_set_flags(channel_in, flags, NULL);
-        flags = g_io_channel_get_flags(channel_err);
-        flags |= G_IO_FLAG_NONBLOCK;
-        g_io_channel_set_flags(channel_err, flags, NULL);
-*/
+
         g_io_channel_set_close_on_unref(channel_in, TRUE);
         g_io_channel_set_close_on_unref(channel_out, TRUE);
         g_io_channel_set_close_on_unref(channel_err, TRUE);
@@ -1043,15 +1061,9 @@ gpointer launch_player(gpointer data)
 #else
         g_timeout_add(1000, thread_query, threaddata);
 #endif
-        if (verbose > 1)
-            printf
-                ("About to lock second time, next message about thread running, should be unlocking\n");
-        // g_mutex_lock(thread_running);
-		g_cond_wait(mplayer_complete_cond,thread_running);
+        g_cond_wait(mplayer_complete_cond, thread_running);
         if (verbose)
             printf("Thread completing\n");
-        if (verbose > 1)
-            printf("thread running, in second lock\n");
         threaddata->done = TRUE;
         g_source_remove(watch_in_id);
         g_source_remove(watch_err_id);
@@ -1101,12 +1113,17 @@ gpointer launch_player(gpointer data)
 #endif
 
         dbus_enable_screensaver();
-
-        if (verbose > 1)
-            printf("thread running, before unlocking second time\n");
         g_mutex_unlock(thread_running);
-        if (verbose > 1)
-            printf("thread running, unlocked second time\n");
+        
+		if (idledata->cachepercent < 0 && g_str_has_prefix(threaddata->filename, "mms")) {
+            dontplaynext = TRUE;
+            playback_error = ERROR_RETRY_WITH_MMSHTTP;
+        }
+
+		if (idledata->cachepercent < 0 && g_str_has_prefix(threaddata->filename, "mmshttp")) {
+            dontplaynext = TRUE;
+            playback_error = ERROR_RETRY_WITH_HTTP;
+        }
 
         if (dontplaynext == FALSE) {
             if (next_item_in_playlist(&iter)) {
@@ -1116,7 +1133,7 @@ gpointer launch_player(gpointer data)
                     g_strlcpy(idledata->info, filename, 4096);
                     g_idle_add(set_media_info, idledata);
                     p = (PlayData *) g_malloc(sizeof(PlayData));
-                    g_strlcpy(p->filename, filename, 4096);
+                    g_strlcpy(p->uri, filename, 4096);
                     p->playlist = playlist;
                     g_idle_add(play, p);
                     g_free(filename);
@@ -1132,7 +1149,7 @@ gpointer launch_player(gpointer data)
                         g_strlcpy(idledata->info, filename, 4096);
                         g_idle_add(set_media_info, idledata);
                         p = (PlayData *) g_malloc(sizeof(PlayData));
-                        g_strlcpy(p->filename, filename, 4096);
+                        g_strlcpy(p->uri, filename, 4096);
                         p->playlist = playlist;
                         g_idle_add(play, p);
                         g_free(filename);
@@ -1140,7 +1157,7 @@ gpointer launch_player(gpointer data)
                 } else {
                     idledata->fullscreen = 0;
                     g_idle_add(set_fullscreen, idledata);
-					g_idle_add(set_stop, idledata);
+                    g_idle_add(set_stop, idledata);
                 }
 
                 if (quit_on_complete) {
@@ -1150,10 +1167,29 @@ gpointer launch_player(gpointer data)
         } else {
             if (playback_error == ERROR_RETRY_WITH_PLAYLIST) {
                 p = (PlayData *) g_malloc(sizeof(PlayData));
-                g_strlcpy(p->filename, threaddata->filename, 4096);
+                g_strlcpy(p->uri, threaddata->filename, 4096);
                 p->playlist = 1;
                 g_idle_add(play, p);
             }
+			
+            if (playback_error == ERROR_RETRY_WITH_HTTP) {
+                p = (PlayData *) g_malloc(sizeof(PlayData));
+				filename = switch_protocol(threaddata->filename,"http");
+                g_strlcpy(p->uri, filename, 4096);
+				g_free(filename);
+                p->playlist = threaddata->playlist;
+                g_idle_add(play, p);
+            }
+
+			if (playback_error == ERROR_RETRY_WITH_MMSHTTP) {
+                p = (PlayData *) g_malloc(sizeof(PlayData));
+				filename = switch_protocol(threaddata->filename,"mmshttp");
+                g_strlcpy(p->uri, filename, 4096);
+				g_free(filename);
+                p->playlist = threaddata->playlist;
+                g_idle_add(play, p);
+            }
+
             dontplaynext = FALSE;
         }
 
